use std::io::Write;

use clap::{arg, Parser};

use crate::fetcher::{DockerTagsFetcher, QueryArgs};
use crate::fetcher::docker_hub::DockerHubTagsFetcher;
use crate::fetcher::docker_registry::{docker_config, RegistryTagsFetcher};

mod fetcher;

#[derive(Parser)]
#[command(name = "dockertags", author, version, about = "List all tags for a Docker image on a remote docker_registry", long_about = "List all tags for a Docker image on a remote docker_registry")]
struct Args {
    /// docker image name
    #[arg(value_name = "REPOSITORY")]
    repository: String,
    /// docker image architecture
    #[arg(short, long, value_name = "ARCHITECTURE")]
    arch: Option<String>,
    /// image tags name to filter
    #[arg(short, long, value_name = "TAG")]
    filter: Option<String>,
}


fn main() {
    let args = Args::parse();
    let query_args = QueryArgs::new(args.repository.as_str(), args.filter, args.arch);
    let config = docker_config::read_config();
    let mut tags = match config {
        None => {
            let fetcher = DockerHubTagsFetcher::new();
            fetcher.get_tags(&query_args)
        }
        Some(x) => {
            let fetcher = RegistryTagsFetcher::new(&x);
            fetcher.get_tags(&query_args)
        }
    };
    tags.sort();
    for tag in tags {
        let _ = writeln!(std::io::stdout(), "{}", &tag);
    }
}
